@@ -38,6 +38,12 @@ Chirp C Client
 #define CHIRP_ERROR_TOO_MANY_OPEN -9
 #define CHIRP_ERROR_BUSY -10
 #define CHIRP_ERROR_TRY_AGAIN -11
+#define CHIRP_ERROR_BAD_FD -12
+#define CHIRP_ERROR_IS_DIR -13
+#define CHIRP_ERROR_NOT_DIR -14
+#define CHIRP_ERROR_NOT_EMPTY -15
+#define CHIRP_ERROR_CROSS_DEVICE_LINK -16
+#define CHIRP_ERROR_OFFLINE -17
 #define CHIRP_ERROR_UNKNOWN -127
 
 #endif
