@@ -44,7 +44,7 @@ int snprintf(char *str, size_t size, const char *format, ...);
 /**	Disable the warning about the number of formal parameters 
 	differing from a previous declaration */
 #pragma warning ( disable : 4273 ) // inconsistent dll linkage
-int vsnprintf(char *str, size_t size, const char *format, va_list args);
+int __cdecl vsnprintf(char *str, size_t size, const char *format, va_list args);
 #pragma warning ( default : 4273 )
 #endif
 
