@@ -29,7 +29,6 @@
 
 #include "classad/classad_distribution.h"
 #include "classad_oldnew.h"
-#include "conversion.h"
 using namespace std;
 
 #include <list>
@@ -139,7 +138,7 @@ Tests::CmdLine( int argc, const char *argv[] )
 
 		if ( arg.Match('d', "debug") ) {
 			if ( arg.hasOpt() ) {
-				set_debug_flags( const_cast<char *>(arg.getOpt()) );
+				set_debug_flags( const_cast<char *>(arg.getOpt()), 0 );
 				argno = arg.ConsumeOpt( );
 			} else {
 				fprintf(stderr, "Value needed for '%s'\n", arg.Arg() );
@@ -452,7 +451,7 @@ void handle_sig(int /*sig*/ )
 int
 main(int argc, const char **argv)
 {
-	DebugFlags = D_ALWAYS;
+	set_debug_flags(NULL, D_ALWAYS);
 
 	set_mySubSystem( "TEST_LEASE_MANAGER", SUBSYSTEM_TYPE_TOOL );
 
@@ -461,8 +460,7 @@ main(int argc, const char **argv)
 	config();
 
 		// Set up the dprintf stuff...
-	Termlog = true;
-	dprintf_config("TEST_LEASE_MANAGER", get_param_functions());
+	dprintf_set_tool_debug("TEST_LEASE_MANAGER", 0);
 
 	int		status;
 	status = tests.CmdLine( argc, argv );
