@@ -49,7 +49,6 @@ my $CONDOR_VACATE_JOB = 'condor_vacate_job';
 my $CONDOR_RESCHD = 'condor_reschedule';
 my $CONDOR_RM = 'condor_rm';
 
-my $DEBUG = 0;
 my $DEBUGLEVEL = 1; # turn on lowest level output
 my $cluster = 0;
 my $num_active_jobs = 0;
@@ -62,7 +61,7 @@ my $timer_time = 0;
 my $TimedCallbackWait = 0;
 my $SubmitCallback;
 my $ExecuteCallback;
-my $EvictedCallback;
+#my $EvictedCallback;
 my $EvictedWithCheckpointCallback;
 my $EvictedWithRequeueCallback;
 my $EvictedWithoutCheckpointCallback;
@@ -71,7 +70,10 @@ my $ExitedSuccessCallback;
 my $ExitedFailureCallback;
 my $ExitedAbnormalCallback;
 my $AbortCallback;
+my $SuspendedCallback;
+my $UnsuspendedCallback;
 my $ShadowCallback;
+my $ImageUpdatedCallback;
 my $HoldCallback;
 my $ReleaseCallback;
 my $DisconnectedCallback;
@@ -96,7 +98,7 @@ sub Reset
 
     undef $SubmitCallback;
     undef $ExecuteCallback;
-    undef $EvictedCallback;
+    #undef $EvictedCallback;
     undef $EvictedWithCheckpointCallback;
     undef $EvictedWithRequeueCallback;
     undef $EvictedWithoutCheckpointCallback;
@@ -106,6 +108,7 @@ sub Reset
     undef $ExitedAbnormalCallback;
     undef $AbortCallback;
     undef $ShadowCallback;
+	undef $ImageUpdatedCallback;
     undef $HoldCallback;
     undef $ReleaseCallback;
     undef $DisconnectedCallback;
@@ -240,9 +243,9 @@ sub TestSubmit
 
     my $debug_arg = "";
 
-    if( $DEBUG ) {
+    #if( $DEBUG ) {
 	$debug_arg = "-debug";
-    }
+    #}
 
     # reset global state
     # Reset();
@@ -452,11 +455,12 @@ sub RegisterExecute
     $ExecuteCallback = $sub;
 }
 
-sub RegisterEvicted
-{
-    my $sub = shift || croak "missing argument";
-    $EvictedCallback = $sub;
-}
+# only spcific evictions are enabled
+#sub RegisterEvicted
+#{
+    #my $sub = shift || croak "missing argument";
+    #$EvictedCallback = $sub;
+#}
 
 sub RegisterEvictedWithCheckpoint
 {
@@ -504,6 +508,11 @@ sub RegisterAbort
     my $sub = shift || croak "missing argument";
     $AbortCallback = $sub;
 }
+sub RegisterImageUpdated
+{
+    my $sub = shift || croak "missing argument";
+    $ImageUpdatedCallback = $sub;
+}
 sub RegisterShadow
 {
     my $sub = shift || croak "missing argument";
@@ -523,11 +532,13 @@ sub RegisterDisconnected
 {
     my $sub = shift || croak "missing argument";
     $DisconnectedCallback = $sub;
+	debug("Registering Disconnnected callback\n",2);
 }
 sub RegisterReconnected
 {
     my $sub = shift || croak "missing argument";
     $ReconnectedCallback = $sub;
+	debug("Registering Reconnnected callback\n",2);
 }
 sub RegisterReconnectFailed
 {
@@ -549,6 +560,20 @@ sub RegisterWantError
     my $sub = shift || croak "missing argument";
     $WantErrorCallback = $sub;
 }
+sub RegisterSuspended
+{
+    my $sub = shift || croak "missing argument";
+    $SuspendedCallback = $sub;
+
+	debug("Registering Suspended callback\n",2);
+}
+sub RegisterUnsuspended
+{
+    my $sub = shift || croak "missing argument";
+    $UnsuspendedCallback = $sub;
+
+	debug("Registering Unsuspended callback\n",2);
+}
 
 sub RegisterTimed
 {
@@ -580,6 +605,17 @@ sub IsAbsolutePath {
 
 # spawn process to monitor the submit log file and execute callbacks
 # upon seeing registered events
+# July 26, 2013
+#
+# Major architecture change as we used more callbacks for testing
+# I stumbled on value in the test which were to be changed by the callbacks were not being
+# changed. What was happening in DoTest was that we would fork and the child was
+# running monitor and that callbacks were changing the variables in the child's copy.
+# The process that is the test calling DoTest was simply waiting for the child to die
+# so it could clean up. Now The test switches to be doing the monitoring and the 
+# callbacks switch it back up to the test code for a bit. The monitor and the 
+# test had always been lock-steped anyways so getting rid of the child
+# has had little change except that call backs can be fully functional now.
 sub Monitor
 {
     my $timeout = shift;
@@ -587,7 +623,6 @@ sub Monitor
     my $line;
     #my %info;
 	my $timestamp = 0;
-    my $parent_pid = getppid();
 
     debug( "Entering Monitor\n" ,5);
 
@@ -664,11 +699,6 @@ sub Monitor
 	while( ! defined $line )
 	{
 		sleep 2;
-		if( getppid() != $parent_pid )
-		{
-		    print "\nCondor::Monitor: our parent pid $parent_pid has gone away.  Aborting.\n";
-		    return 0;
-		}
 
 		if(defined $TimedCallback)
 		{
@@ -687,7 +717,7 @@ sub Monitor
 	# if this line is for another cluster, ignore
 	if ( $line =~ /^\d+\s+\(0*(\d+)\./ && $1 != $cluster )
 	{
-	    debug( "log line for cluster $1, not $cluster -- ignoring...\n" ,1);
+	    #debug( "log line for cluster $1, not $cluster -- ignoring...\n" ,1);
 	    next LINE;
 	}
 	
@@ -699,10 +729,6 @@ sub Monitor
 
 	    debug( "Saw job ($1.$2) evicted\n" ,2);
 
-	    # execute callback if one is registered
-	    &$EvictedCallback( %info )
-		if defined $EvictedCallback;
-
 	    # read next line to see if job was checkpointed (but first
 	    # sleep for 5 seconds to give it a chance to appear)
 	    # [this should really loop like above, not sleep like this...]
@@ -747,9 +773,16 @@ sub Monitor
 		debug( "parse error on line $linenum of $info{'log'}:\n" .
 		       "   no checkpoint message found after eviction: " .
 		       "continuing...\n" ,1);
+		debug( "Eviction type expected:<$line>\n", 1);
 		# re-parse line so we don't miss whatever it said
 		goto PARSE;
 	    }
+		# let all the special callbacks got first
+	    # execute callback if one is registered
+		# Are there any other types of evictions?
+	    #&$EvictedCallback( %info )
+		#if defined $EvictedCallback;
+
 	    next LINE;
 	}
 
@@ -851,6 +884,64 @@ sub Monitor
 	    next LINE;
 	}
 
+	# 006: Image Size Updated
+	if( $line =~ /^006\s+\(0*(\d+)\.0*(\d+).*?Image\s+size\s+of\s+job\s+updated:\s+(\d+)\s*$/ )
+	{
+	    $info{'cluster'} = $1;
+	    $info{'job'} = $2;
+		$info{'imagesize'} = $3;
+
+	    #debug( "Saw Image Size Update <$3>\n" ,2);
+		#print "Saw Image Size Update <$3>\n";
+
+	    # read next line to see current Megs
+	    $line = <SUBMIT_LOG>;
+		while( ! defined $line )
+		{
+			sleep 2;
+			if(defined $TimedCallback)
+			{
+				CheckTimedCallback();
+			}
+			$line = <SUBMIT_LOG>;
+		}
+	    CondorUtils::fullchomp($line);
+	    $linenum++;
+
+		# Grab current memory in Megs
+		if($line =~ /^\s*(\d+).*$/) {
+			$info{'megs'} = $1;
+			debug( "Memory usage $1 (MB)\n",2);
+		}
+
+	    # read next line to see current RSS
+	    $line = <SUBMIT_LOG>;
+		while( ! defined $line )
+		{
+			sleep 2;
+			if(defined $TimedCallback)
+			{
+				CheckTimedCallback();
+			}
+			$line = <SUBMIT_LOG>;
+		}
+	    CondorUtils::fullchomp($line);
+	    $linenum++;
+
+		# Grab current memory in Megs
+		if($line =~ /^\s*(\d+).*$/) {
+			$info{'rss'} = $1;
+			debug( "RSS usage $1 (K)\n",2);
+		}
+
+		# execute callback if one is registered
+		&$ImageUpdatedCallback( %info )
+		    if defined $ImageUpdatedCallback;
+
+	    next LINE;
+
+	}
+
 	# 007: shadow exception
 	if( $line =~ /^007\s+\(0*(\d+)\.0*(\d+)/ )
 	{
@@ -897,7 +988,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw Job Disconnected\n" ,2);
+	    debug( "Saw Job Disconnected\n" ,1);
 
 	    # read next line to see cause
 	    $line = <SUBMIT_LOG>;
@@ -944,7 +1035,7 @@ sub Monitor
 	    $info{'cluster'} = $1;
 	    $info{'job'} = $2;
 
-	    debug( "Saw Job Reconnected\n" ,2);
+	    debug( "Saw Job Reconnected\n" ,1);
 
 	    # read next line to see cause
 	    $line = <SUBMIT_LOG>;
@@ -992,6 +1083,7 @@ sub Monitor
 	    $info{'job'} = $2;
 
 	    debug( "Saw Job Reconnect Fail\n" ,2);
+		print "Saw reconnect fail\n";
 
 	    # read next line to see cause
 	    $line = <SUBMIT_LOG>;
@@ -1026,8 +1118,54 @@ sub Monitor
 		$info{'recovery'} = $line;
 
 		# execute callback if one is registered
-		&$ReconnectedCallback( %info )
-		    if defined $ReconnectedCallback;
+		&$ReconnectFailedCallback( %info )
+		    if defined $ReconnectFailedCallback;
+
+	    next LINE;
+	}
+
+	# 010: Job Suspended
+	if( $line =~ /^010\s+\(0*(\d+)\.0*(\d+)/ )
+	{
+	    $info{'cluster'} = $1;
+	    $info{'job'} = $2;
+
+	    debug( "Saw Job Suspended\n" ,1);
+
+	    # read next line to see processes affected
+	    $line = <SUBMIT_LOG>;
+		while( ! defined $line )
+		{
+			sleep 2;
+			if(defined $TimedCallback)
+			{
+				CheckTimedCallback();
+			}
+			$line = <SUBMIT_LOG>;
+		}
+	    CondorUtils::fullchomp($line);
+	    $linenum++;
+
+		$info{'processes'} = $line;
+
+		# execute callback if one is registered
+		&$SuspendedCallback( %info )
+		    if defined $SuspendedCallback;
+
+	    next LINE;
+	}
+
+	# 010: Job Unsuspended
+	if( $line =~ /^011\s+\(0*(\d+)\.0*(\d+)/ )
+	{
+	    $info{'cluster'} = $1;
+	    $info{'job'} = $2;
+
+	    debug( "Saw Job Unsuspended\n" ,1);
+
+		# execute callback if one is registered
+		&$UnsuspendedCallback( %info )
+		    if defined $UnsuspendedCallback;
 
 	    next LINE;
 	}
@@ -1184,8 +1322,11 @@ sub CheckTimedCallback
 	{
 		#call timed callback
 		debug("Called timed callback!!!!!!-- $cluster --\n",5);
-		&$TimedCallback( %info )
-			if defined $TimedCallback;
+		if(defined $TimedCallback) {
+			my $tempcallback = $TimedCallback; # save and removed callback request
+			RemoveTimed();
+			&$tempcallback(%info);
+		}
 	}
 }
 
@@ -1206,7 +1347,6 @@ sub CheckTimedCallback
 ################################################################################
 
 sub debug {
-    return unless $DEBUG;
     my ($msg, $level) = @_;
     
     if(!(defined $level)) {
@@ -1218,15 +1358,10 @@ sub debug {
 }
 
 sub DebugLevel {
-    $DEBUGLEVEL = shift;
-}
-
-sub DebugOn {
-    $DEBUG = 1;
-}
-
-sub DebugOff {
-    $DEBUG = 0;
+	my $newlevel = shift;
+	my $oldlevel = $DEBUGLEVEL;
+    $DEBUGLEVEL = $newlevel;
+	return($oldlevel);
 }
 
 sub ParseSubmitFile
@@ -1297,7 +1432,7 @@ sub ParseSubmitFile
 }
 
 sub timestamp {
-    return strftime("%Y/%m/%d %H:%M:%S", localtime);
+    return strftime("%H:%M:%S", localtime);
 }
 
 sub safe_WIFEXITED {
