@@ -66,9 +66,12 @@ MachAttributes::MachAttributes()
 		// Number of CPUs.  Since this is used heavily by the ResMgr
 		// instantiation and initialization, we need to have a real
 		// value for this as soon as the MachAttributes object exists.
-	m_num_cpus = sysapi_ncpus();
+	bool count_hyper = param_boolean("COUNT_HYPERTHREAD_CPUS", true);
+	int ncpus=0, nhyper_cpus=0;
+	sysapi_ncpus_raw(&ncpus, &nhyper_cpus);
+	m_num_real_cpus = count_hyper ? nhyper_cpus : ncpus;
+	m_num_cpus = param_integer("NUM_CPUS");
 
-	m_num_real_cpus = sysapi_ncpus_raw();
 
 		// The same is true of physical memory.  If we had an error in
 		// sysapi_phys_memory(), we need to just EXCEPT with a message
@@ -448,7 +451,7 @@ MachAttributes::compute( amask_t how_much )
 	if( IS_UPDATE(how_much) && IS_SHARED(how_much) ) {
 
 		m_virt_mem = sysapi_swap_space();
-		dprintf( D_FULLDEBUG, "Swap space: %lu\n", m_virt_mem );
+		dprintf( D_FULLDEBUG, "Swap space: %lld\n", m_virt_mem );
 
 #if defined(WIN32)
 		credd_test();
@@ -516,137 +519,258 @@ MachAttributes::compute( amask_t how_much )
 
 }
 
+// Allocate a user-defined resource deviceid to a slot
+// when assign_to_sub > 0, then assign an unused resource from
+// slot assigned_to to it's child dynamic slot assign_to_sub.
+//
+const char * MachAttributes::AllocateDevId(const std::string & tag, int assign_to, int assign_to_sub)
+{
+	if ( ! assign_to) return NULL;
 
-void MachAttributes::init_machine_resources() {
-    // defines the space of local machine resources, and their quantity
-    m_machres_map.clear();
+	slotres_devIds_map_t::const_iterator f(m_machres_devIds_map.find(tag));
+	if (f !=  m_machres_devIds_map.end()) {
+		const slotres_assigned_ids_t & ids(f->second);
+		slotres_assigned_id_owners_t & owners = m_machres_devIdOwners_map[tag];
+		while (owners.size() < ids.size()) { owners.push_back(0); }
 
-    // this may be filled from resource inventory scripts
-    m_machres_attr.Clear();
-
-    // these are reserved for standard/fixed resources
-    std::set<string> fixedRes;
-    fixedRes.insert("cpu");
-    fixedRes.insert("cpus");
-    fixedRes.insert("disk");
-    fixedRes.insert("swap");
-    fixedRes.insert("mem");
-    fixedRes.insert("memory");
-    fixedRes.insert("ram");
-
-    // get the list of defined local resource names:
-    char* ptmp = param("MACHINE_RESOURCE_NAMES");
-    string resnames;
-    if (NULL != ptmp) {
-        // if admin defined MACHINE_RESOURCE_NAMES, then use this list
-        // as the source of expected custom machine resources
-        resnames = ptmp;
-        free(ptmp);
-    } else {
-        // otherwise, build the list of custom machine resources from
-        // all occurrences of MACHINE_RESOURCE_<resname>
-        std::set<string> resset;
-        Regex re;
-        int err = 0;
-        const char* pszMsg = 0;
-        const string prefix = "MACHINE_RESOURCE_";
-        const string invprefix = "INVENTORY_";
-        const string restr = prefix + "(.+)";
-        ASSERT(re.compile(restr.c_str(), &pszMsg, &err, PCRE_CASELESS));
-	std::vector<std::string> resdef;
-        const int n = param_names_matching(re, resdef);
-        for (int j = 0;  j < n;  ++j) {
-            string rname = resdef[j].substr(prefix.length());
-            string testinv = rname.substr(0, invprefix.length());
-            if (0 == strcasecmp(testinv.c_str(), invprefix.c_str())) {
-                // if something was defined via MACHINE_RESOURCE_INVENTORY_<rname>, strip "INVENTORY_"
-                rname = rname.substr(invprefix.length());
-            }
-            std::transform(rname.begin(), rname.end(), rname.begin(), ::tolower);
-            resset.insert(rname);
-        }
-        for (std::set<string>::iterator j(resset.begin());  j != resset.end();  ++j) {
-            resnames += " ";
-            resnames += *j;
-        }
-    }
-
-    // scan the list of local resources, to obtain their quantity and associated attributes (if any)
-    StringList reslist(resnames.c_str());
-    reslist.rewind();
-    while (const char* rnp = reslist.next()) {
-        string rname(rnp);
-        std::transform(rname.begin(), rname.end(), rname.begin(), ::tolower);
-        if (fixedRes.count(rname) > 0) {
-            EXCEPT("pre-defined resource name '%s' is reserved", rname.c_str());
-        }
-
-        // If MACHINE_RESOURCE_<rname> is present, use that and move on:
-        string pname;
-        formatstr(pname, "MACHINE_RESOURCE_%s", rname.c_str());
-        char* machresp = param(pname.c_str());
-        if (machresp) {
-            int v = param_integer(pname.c_str(), 0, 0, INT_MAX);
-            m_machres_map[rname] = v;
-            free(machresp);
-            continue;
-        }
-
-        // current definition of REMIND macro is not working with gcc
-        #pragma message("MACHINE_RESOURCE_INVENTORY_<rname> is deprecated, and will be removed when a solution using '|' in config files is fleshed out")
-        // if we didn't find MACHINE_RESOURCE_<rname>, then try MACHINE_RESOURCE_INVENTORY_<rname>
-        formatstr(pname, "MACHINE_RESOURCE_INVENTORY_%s", rname.c_str());
-        char* invscriptp = param(pname.c_str());
-        if (NULL == invscriptp) {
-            EXCEPT("Missing configuration for local machine resource %s", rname.c_str());
-        }
-
-        // if there is an inventory script, then attempt to execute it and use its output to
-        // identify the local resource quantity, and any associated attributes:
-        string invscript = invscriptp;
-        free(invscriptp);
-
-        ArgList invcmd;
-        StringList invcmdal(invscript.c_str());
-        invcmdal.rewind();
-        while (char* invarg = invcmdal.next()) {
-            invcmd.AppendArg(invarg);
-        }
-        FILE* fp = my_popen(invcmd, "r", FALSE);
-        if (NULL == fp) {
-            EXCEPT("Failed to execute local resource inventory script \"%s\"\n", invscript.c_str());
-        }
-
-        ClassAd invad;
-        char invline[1000];
-        while (fgets(invline, sizeof(invline), fp)) {
-            if (!invad.Insert(invline)) {
-                dprintf(D_ALWAYS, "Failed to insert \"%s\" into machine resource attributes: ignoring\n", invline);
-            }
-        }
-        my_pclose(fp);
-
-        // require the detected machine resource to be present as an attribute
-        string ccname(rname.c_str());
-        *(ccname.begin()) = toupper(*(ccname.begin()));
-        string detname;
-        formatstr(detname, "%s%s", ATTR_DETECTED_PREFIX, ccname.c_str());
-        int v = 0;
-        if (!invad.LookupInteger(detname.c_str(), v)) {
-            EXCEPT("Missing required attribute \"%s = <n>\" from output of %s\n", detname.c_str(),  invscript.c_str());
-        }
-
-        // success
-        m_machres_map[rname] = v;
-        invad.Delete(rname.c_str());
-        m_machres_attr.Update(invad);
-    }
-
-    for (slotres_map_t::iterator j(m_machres_map.begin());  j != m_machres_map.end();  ++j) {
-        dprintf(D_ALWAYS, "Local machine resource %s = %d\n", j->first.c_str(), int(j->second));
-    }
+		// when the sub-id is positive, that means we are binding a dynamic slot
+		// in that case we want to use only dev-ids that are already owned by the partitionable slot.
+		// but not yet assigned to any of  it's dynamic children.
+		//
+		if (assign_to_sub > 0) {
+			for (int ii = 0; ii < (int)ids.size(); ++ii) {
+				if (owners[ii].id == assign_to && owners[ii].dyn_id == 0) {
+					owners[ii].dyn_id = assign_to_sub;
+					return ids[ii].c_str();
+				}
+			}
+		} else {
+			for (int ii = 0; ii < (int)ids.size(); ++ii) {
+				if ( ! owners[ii].id) {
+					owners[ii] = FullSlotId(assign_to, assign_to_sub);
+					return ids[ii].c_str();
+				}
+			}
+		}
+	}
+	return NULL;
 }
 
+bool MachAttributes::ReleaseDynamicDevId(const std::string & tag, const char * id, int was_assigned_to, int was_assign_to_sub)
+{
+	slotres_devIds_map_t::const_iterator f(m_machres_devIds_map.find(tag));
+	if (f !=  m_machres_devIds_map.end()) {
+		const slotres_assigned_ids_t & ids(f->second);
+		slotres_assigned_id_owners_t & owners = m_machres_devIdOwners_map[tag];
+		while (owners.size() < ids.size()) { owners.push_back(0); }
+
+		for (int ii = 0; ii < (int)ids.size(); ++ii) {
+			if (id == ids[ii]) {
+				if (owners[ii].id == was_assigned_to && owners[ii].dyn_id == was_assign_to_sub) {
+					owners[ii].dyn_id = 0;
+					return true;
+				}
+				break;
+			}
+		}
+	}
+	return false;
+}
+
+// res_value is a string that contains either a number, which is the count of a
+// fungable resource, or a list of ids of non-fungable resources.
+// if res_value contains a list, then ids is set on exit from this function
+// otherwise, ids empty.
+static double parse_user_resource_config(const char * tag, const char * res_value, StringList & ids)
+{
+	ids.clearAll();
+	double num = 0;
+	char * pend = NULL;
+	num = strtod(res_value, &pend);
+	if (pend != res_value) { while(isspace(*pend)) { ++pend; } }
+	bool is_simple_double = (pend && pend != res_value && !*pend);
+	if ( ! is_simple_double) {
+		// ok not a simple double, try evaluating it as a classad expression.
+		ClassAd ad;
+		if (ad.AssignExpr(tag,res_value) && ad.EvalFloat(tag, NULL, num)) {
+			// it was an expression that evaluated to a double, so it's a simple double after all
+			is_simple_double = true;
+		} else {
+			// not a simple double, so treat it as a list of id's, the number of resources is the number of ids
+			ids.initializeFromString(res_value);
+			num = ids.number();
+		}
+	}
+
+	return num;
+}
+
+// run a script, and take use its output to configure a custom resource of type tag
+//
+int MachAttributes::init_machine_resource_from_script(const char * tag, const char * script_cmd) {
+
+	ArgList args;
+	MyString errors;
+	if(!args.AppendArgsV1RawOrV2Quoted(script_cmd, &errors)) {
+		printf("Can't append cmd %s(%s)\n", script_cmd, errors.Value());
+		return -1;
+	}
+
+	int quantity = 0;
+	int offline = 0;
+
+	FILE * fp = my_popen(args, "r", FALSE);
+	if ( ! fp) {
+		//PRAGMA_REMIND("tj: should failure to run a res inventory script really bring down the startd?")
+		EXCEPT("Failed to execute local resource '%s' inventory script \"%s\"\n", tag, script_cmd);
+	} else {
+		int error = 0;
+		bool is_eof = false;
+		ClassAd ad;
+		int cAttrs = ad.InsertFromFile(fp, is_eof, error);
+		if (cAttrs <= 0) {
+			if (error) dprintf(D_ALWAYS, "Could not parse ClassAd for local resource '%s' (error %d) assuming quantity of 0\n", tag, error);
+		} else {
+			MyString attr(ATTR_OFFLINE_PREFIX); attr += tag;
+			MyString res_value;
+			StringList offline_ids;
+			if (ad.LookupString(attr.c_str(),res_value)) {
+				offline = parse_user_resource_config(tag, res_value.c_str(), offline_ids);
+			} else {
+				attr = "OFFLINE_MACHINE_RESOURCE_"; attr += tag;
+				if (param(res_value, attr.c_str()) && !res_value.empty()) {
+					offline = parse_user_resource_config(tag, res_value.c_str(), offline_ids);
+				}
+			}
+
+			attr = ATTR_DETECTED_PREFIX; attr += tag;
+			if (ad.LookupString(attr.c_str(),res_value)) {
+				StringList ids;
+				quantity = parse_user_resource_config(tag, res_value.c_str(), ids);
+				if ( ! ids.isEmpty()) {
+					offline = 0; // we only want to count ids that match the detected list
+					ids.rewind();
+					while (const char* id = ids.next()) {
+						if (offline_ids.contains(id)) {
+							this->m_machres_offline_devIds_map[tag].push_back(id);
+							++offline;
+						} else {
+							this->m_machres_devIds_map[tag].push_back(id);
+						}
+					}
+				}
+			}
+
+			// make sure that the inventory ad doesn't have an attribute for the tag name
+			// i.e. if the inventory is GPUS, then DetectedGPUs=N is required, but it cant have GPUs=N
+			ad.Delete(tag);
+			m_machres_attr.Update(ad);
+		}
+		my_pclose(fp);
+	}
+
+	return quantity - offline;
+}
+
+// this static callback for the param param iteration using foreach_param_*
+// is called once for each param matching MACHINE_RESOURCE_*
+// it extracts the resource tag from the param name, then fetches the resource quantity
+// and initializes the m_machres_map 
+bool MachAttributes::init_machine_resource(MachAttributes * pme, HASHITER & it) {
+	const char * name = hash_iter_key(it);
+	const char * rawval = hash_iter_value(it);
+	if ( ! rawval || ! rawval[0])
+		return true; // keep iterating
+
+	char * res_value = param(name);
+	if ( ! res_value)
+		return true; // keep iterating
+
+	double num = 0;
+	double offline = 0;
+	const char * tag = name + sizeof("MACHINE_RESOURCE_")-1;
+
+	// MACHINE_RESOURCE_INVENTORY_* is a special case that runs a script that generates a classad.
+	if (starts_with_ignore_case(tag, "INVENTORY_")) {
+		tag += sizeof("INVENTORY_")-1;
+		if (res_value) {
+			num = pme->init_machine_resource_from_script(tag, res_value);
+		}
+	} else {
+		StringList offline_ids;
+		MyString off_name("OFFLINE_"); off_name += name;
+		MyString my_value;
+		if (param(my_value, off_name.c_str()) && !my_value.empty()) {
+			offline = parse_user_resource_config(tag, my_value.c_str(), offline_ids);
+		}
+
+		// if the param value parses as a double, then we can handle it simply
+		StringList ids;
+		num = parse_user_resource_config(tag, res_value, ids);
+		if ( ! ids.isEmpty()) {
+			offline = 0; // we only want to count ids that match the detected list
+			ids.rewind();
+			while (const char* id = ids.next()) {
+				if (offline_ids.contains(id)) {
+					pme->m_machres_offline_devIds_map[tag].push_back(id);
+					++offline;
+				} else {
+					pme->m_machres_devIds_map[tag].push_back(id);
+				}
+			}
+		}
+	}
+
+	if (num < 0) num = 0;
+	pme->m_machres_map[tag] = num - offline;
+	free(res_value);
+
+	return true; // keep iterating
+}
+
+
+void MachAttributes::init_machine_resources() {
+	// defines the space of local machine resources, and their quantity
+	m_machres_map.clear();
+	// defines which local machine resource device IDs are in use
+	m_machres_devIds_map.clear();
+	m_machres_offline_devIds_map.clear();
+
+	// this may be filled from resource inventory scripts
+	m_machres_attr.Clear();
+
+	Regex re;
+	int err = 0;
+	const char* pszMsg = 0;
+	ASSERT(re.compile("^MACHINE_RESOURCE_[a-zA-Z0-9_]+", &pszMsg, &err, PCRE_CASELESS));
+	const int iter_options = HASHITER_NO_DEFAULTS; // we can speed up iteration if there are no machine resources in the defaults table.
+	foreach_param_matching(re, iter_options, (bool(*)(void*,HASHITER&))MachAttributes::init_machine_resource, this);
+
+	StringList allowed;
+	char* allowed_names = param("MACHINE_RESOURCE_NAMES");
+	if (allowed_names && allowed_names[0]) {
+		// if admin defined MACHINE_RESOURCE_NAMES, then use this list
+		// as the source of expected custom machine resources
+		allowed.initializeFromString(allowed_names);
+		free(allowed_names);
+	}
+
+	StringList disallowed("CPU CPUS DISK SWAP MEM MEMORY RAM");
+
+	for (slotres_map_t::iterator it(m_machres_map.begin());  it != m_machres_map.end();  ++it) {
+		const char * tag = it->first.c_str();
+		if ( ! allowed.isEmpty() && ! allowed.contains_anycase(tag)) {
+			dprintf(D_ALWAYS, "Local machine resource %s is not in MACHINE_RESOURCE_NAMES so it will have no effect\n", tag);
+			m_machres_map.erase(tag);
+			continue;
+		}
+		if ( ! disallowed.isEmpty() && disallowed.contains_anycase(tag)) {
+			EXCEPT("fatal error - MACHINE_RESOURCE_%s is invalid, '%s' is a reserved resource name\n", tag, tag);
+			continue;
+		}
+		dprintf(D_ALWAYS, "Local machine resource %s = %g\n", tag, it->second);
+	}
+}
 
 void
 MachAttributes::final_idle_dprintf()
@@ -801,7 +925,7 @@ MachAttributes::publish( ClassAd* cp, amask_t how_much)
 
 	if( IS_UPDATE(how_much) || IS_PUBLIC(how_much) ) {
 
-		cp->Assign( ATTR_TOTAL_VIRTUAL_MEMORY, (int)m_virt_mem );
+		cp->Assign( ATTR_TOTAL_VIRTUAL_MEMORY, m_virt_mem );
 
 		cp->Assign( ATTR_TOTAL_CPUS, m_num_cpus );
 
@@ -822,20 +946,37 @@ MachAttributes::publish( ClassAd* cp, amask_t how_much)
 		}
 #endif
 
-        string machine_resources = "cpus memory disk swap";
-        // publish any local resources
-        for (slotres_map_t::iterator j(m_machres_map.begin());  j != m_machres_map.end();  ++j) {
-            string rname(j->first.c_str());
-            *(rname.begin()) = toupper(*(rname.begin()));
-            string attr;
-            formatstr(attr, "%s%s", ATTR_DETECTED_PREFIX, rname.c_str());
-            cp->Assign(attr.c_str(), int(j->second));
-            formatstr(attr, "%s%s", ATTR_TOTAL_PREFIX, rname.c_str());
-            cp->Assign(attr.c_str(), int(j->second));
-            machine_resources += " ";
-            machine_resources += j->first;
-        }
-        cp->Assign(ATTR_MACHINE_RESOURCES, machine_resources.c_str());
+		string machine_resources = "Cpus Memory Disk Swap";
+		// publish any local resources
+		for (slotres_map_t::iterator j(m_machres_map.begin());  j != m_machres_map.end();  ++j) {
+			const char * rname = j->first.c_str();
+			string attr(ATTR_DETECTED_PREFIX); attr += rname;
+			double ipart, fpart = modf(j->second, &ipart);
+			if (fpart >= 0.0 && fpart <= 0.0) {
+				cp->Assign(attr.c_str(), (long long)ipart);
+			} else {
+				cp->Assign(attr.c_str(), j->second);
+			}
+			attr = ATTR_TOTAL_PREFIX; attr += rname;
+			if (fpart >= 0.0 && fpart <= 0.0) {
+				cp->Assign(attr.c_str(), (long long)ipart);
+			} else {
+				cp->Assign(attr.c_str(), j->second);
+			}
+			// are there any offline ids for this resource?
+			slotres_devIds_map_t::const_iterator k = m_machres_offline_devIds_map.find(j->first);
+			if (k != m_machres_offline_devIds_map.end()) {
+				if ( ! k->second.empty()) {
+					attr = ATTR_OFFLINE_PREFIX; attr += k->first;
+					string ids;
+					join(k->second, ",", ids);
+					cp->Assign(attr.c_str(), ids.c_str());
+				}
+			}
+			machine_resources += " ";
+			machine_resources += j->first;
+			}
+		cp->Assign(ATTR_MACHINE_RESOURCES, machine_resources.c_str());
 	}
 
 		// We don't want this inserted into the public ad automatically
@@ -1008,17 +1149,18 @@ MachAttributes::credd_test()
 
 CpuAttributes::CpuAttributes( MachAttributes* map_arg, 
 							  int slot_type,
-							  int num_cpus_arg, 
+							  double num_cpus_arg,
 							  int num_phys_mem,
-							  float virt_mem_fraction,
-							  float disk_fraction,
-                              const slotres_map_t& slotres_map,
+							  double virt_mem_fraction,
+							  double disk_fraction,
+							  const slotres_map_t& slotres_map,
 							  MyString &execute_dir,
 							  MyString &execute_partition_id )
 {
 	map = map_arg;
 	c_type = slot_type;
 	c_num_slot_cpus = c_num_cpus = num_cpus_arg;
+	c_allow_fractional_cpus = num_cpus_arg > 0 && num_cpus_arg < 0.9;
 	c_slot_mem = c_phys_mem = num_phys_mem;
 	c_virt_mem_fraction = virt_mem_fraction;
 	c_disk_fraction = disk_fraction;
@@ -1037,7 +1179,6 @@ CpuAttributes::CpuAttributes( MachAttributes* map_arg,
 	rip = NULL;
 }
 
-
 void
 CpuAttributes::attach( Resource* res_ip )
 {
@@ -1045,16 +1186,63 @@ CpuAttributes::attach( Resource* res_ip )
 }
 
 
+void
+CpuAttributes::bind_DevIds(int slot_id, int slot_sub_id) // bind non-fungable resource ids to a slot
+{
+	if ( ! map)
+		return;
+
+	for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
+		int cAssigned = int(j->second);
+
+		// if this resource already has bound ids, don't bind again.
+		slotres_devIds_map_t::const_iterator m(c_slotres_ids_map.find(j->first));
+		if (m != c_slotres_ids_map.end() && cAssigned == (int)m->second.size())
+			continue;
+
+		slotres_devIds_map_t::const_iterator k(map->machres_devIds().find(j->first));
+		if (k != map->machres_devIds().end()) {
+			for (int ii = 0; ii < cAssigned; ++ii) {
+				const char * id = map->AllocateDevId(j->first, slot_id, slot_sub_id);
+				if ( ! id) {
+					EXCEPT("Failed to bind local resource '%s' \n", j->first.c_str());
+				} else {
+					c_slotres_ids_map[j->first].push_back(id);
+				}
+			}
+		}
+	}
+
+}
+
+void
+CpuAttributes::unbind_DevIds(int slot_id, int slot_sub_id) // release non-fungable resource ids
+{
+	if ( ! map) return;
+	if ( ! slot_sub_id) return;
+
+	for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
+		slotres_devIds_map_t::const_iterator k(c_slotres_ids_map.find(j->first));
+		if (k != c_slotres_ids_map.end()) {
+			slotres_assigned_ids_t & ids = c_slotres_ids_map[j->first];
+			while ( ! ids.empty()) {
+				map->ReleaseDynamicDevId(j->first, ids.back().c_str(), slot_id, slot_sub_id);
+				ids.pop_back();
+			}
+		}
+	}
+}
+
 void
 CpuAttributes::publish( ClassAd* cp, amask_t how_much )
 {
 	if( IS_UPDATE(how_much) || IS_PUBLIC(how_much) ) {
 
-		cp->Assign( ATTR_VIRTUAL_MEMORY, (int)c_virt_mem );
+		cp->Assign( ATTR_VIRTUAL_MEMORY, c_virt_mem );
 
-		cp->Assign( ATTR_TOTAL_DISK, (int)c_total_disk );
+		cp->Assign( ATTR_TOTAL_DISK, c_total_disk );
 
-		cp->Assign( ATTR_DISK, (int)c_disk );
+		cp->Assign( ATTR_DISK, c_disk );
 		
 	}
 
@@ -1064,37 +1252,43 @@ CpuAttributes::publish( ClassAd* cp, amask_t how_much )
 
 		cp->Assign( ATTR_LOAD_AVG, rint((c_owner_load + c_condor_load) * 100) / 100.0 );
 
-		cp->Assign( ATTR_KEYBOARD_IDLE, (int)c_idle );
+		cp->Assign( ATTR_KEYBOARD_IDLE, c_idle );
   
 			// ConsoleIdle cannot be determined on all platforms; thus, only
 			// advertise if it is not -1.
 		if( c_console_idle != -1 ) {
-			cp->Assign( ATTR_CONSOLE_IDLE, (int)c_console_idle );
+			cp->Assign( ATTR_CONSOLE_IDLE, c_console_idle );
 		}
 	}
 
 	if( IS_STATIC(how_much) || IS_PUBLIC(how_much) ) {
 
 		cp->Assign( ATTR_MEMORY, c_phys_mem );
-
-		cp->Assign( ATTR_CPUS, c_num_cpus );
-		
 		cp->Assign( ATTR_TOTAL_SLOT_MEMORY, c_slot_mem );
-		
-		cp->Assign( ATTR_TOTAL_SLOT_DISK, (int)c_slot_disk );
+		cp->Assign( ATTR_TOTAL_SLOT_DISK, c_slot_disk );
 
-		cp->Assign( ATTR_TOTAL_SLOT_CPUS, c_num_slot_cpus );
+		if (c_allow_fractional_cpus) {
+			cp->Assign( ATTR_CPUS, c_num_cpus );
+			cp->Assign( ATTR_TOTAL_SLOT_CPUS, c_num_slot_cpus );
+		} else {
+			cp->Assign( ATTR_CPUS, (int)(c_num_cpus + 0.1) );
+			cp->Assign( ATTR_TOTAL_SLOT_CPUS, (int)(c_num_slot_cpus + 0.1) );
+		}
 		
-        // publish local resource quantities for this slot
-        for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
-            string rname(j->first.c_str());
-            *(rname.begin()) = toupper(*(rname.begin()));
-            string attr;
-            formatstr(attr, "%s%s", "", rname.c_str());
-            cp->Assign(attr.c_str(), int(j->second));
-            formatstr(attr, "%s%s", ATTR_TOTAL_SLOT_PREFIX, rname.c_str());
-            cp->Assign(attr.c_str(), int(c_slottot_map[j->first]));
-        }
+		// publish local resource quantities for this slot
+		for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
+			cp->Assign(j->first.c_str(), int(j->second));
+			string attr = ATTR_TOTAL_SLOT_PREFIX; attr += j->first;
+			cp->Assign(attr.c_str(), int(c_slottot_map[j->first]));
+			slotres_devIds_map_t::const_iterator k(c_slotres_ids_map.find(j->first));
+			if (k != c_slotres_ids_map.end()) {
+				attr = "Assigned";
+				attr += j->first;
+				string ids;
+				join(k->second, ",", ids);
+				cp->Assign(attr.c_str(), ids.c_str());
+			}
+		}
 	}
 }
 
@@ -1125,8 +1319,8 @@ CpuAttributes::compute( amask_t how_much )
 		}
 
 		val = c_total_disk * c_disk_fraction;
-		c_disk = (int)floor( val );
-		if (0 == (int)c_slot_disk)
+		c_disk = (long long)floor(val);
+		if (0 == (long long)c_slot_disk)
 		{
 		  // only use the 1st compute ignore subsequent.
 		  c_slot_disk = c_disk; 
@@ -1149,15 +1343,15 @@ CpuAttributes::display( amask_t how_much )
 				 "CondorLoad:", c_condor_load,
 				 "OwnerLoad:", c_owner_load );
 	} else {
-		if( IsDebugLevel( D_LOAD ) ) {
-			dprintf( D_FULLDEBUG, 
+		if( IsDebugVerbose( D_LOAD ) ) {
+			dprintf( D_LOAD | D_VERBOSE,
 					 "%s %.2f  %s %.2f  %s %.2f\n",  
 					 "SystemLoad:", c_condor_load + c_owner_load,
 					 "CondorLoad:", c_condor_load,
 					 "OwnerLoad:", c_owner_load );
 		}
-		if( IsDebugLevel( D_KEYBOARD ) ) {
-			dprintf( D_FULLDEBUG, 
+		if( IsDebugVerbose( D_KEYBOARD ) ) {
+			dprintf( D_KEYBOARD | D_VERBOSE,
 					 "Idle time: %s %-8d %s %d\n",  
 					 "Keyboard:", (int)c_idle, 
 					 "Console:", (int)c_console_idle );
@@ -1173,15 +1367,15 @@ CpuAttributes::show_totals( int dflag )
 			 "slot type %d: " , c_type);
 
 	
-	if( c_num_cpus == AUTO_CPU ) {
+	if (IS_AUTO_SHARE(c_num_cpus)) {
 		::dprintf( dflag | D_NOHEADER, 
 			 "Cpus: auto");
 	} else {
 		::dprintf( dflag | D_NOHEADER, 
-			 "Cpus: %d", c_num_cpus);
+			 "Cpus: %f", c_num_cpus);
 	}
 
-	if( c_phys_mem == AUTO_MEM ) {
+	if (IS_AUTO_SHARE(c_phys_mem)) {
 		::dprintf( dflag | D_NOHEADER, 
 				   ", Memory: auto" );
 	}
@@ -1234,6 +1428,27 @@ CpuAttributes::dprintf( int flags, const char* fmt, ... )
 	va_end( args );
 }
 
+void
+CpuAttributes::swap_attributes(CpuAttributes & attra, CpuAttributes & attrb, int flags)
+{
+	if (flags & 1) {
+		// swap execute directories.
+		MyString str = attra.c_execute_dir;
+		attra.c_execute_dir = attrb.c_execute_dir;
+		attrb.c_execute_dir = str;
+
+		// swap execute partition ids
+		str = attra.c_execute_partition_id;
+		attra.c_execute_partition_id = attrb.c_execute_partition_id;
+		attrb.c_execute_partition_id = str;
+
+		// swap total disk since we swapped partition ids
+		long long ll = attra.c_total_disk;
+		attra.c_total_disk = attrb.c_total_disk;
+		attrb.c_total_disk = ll;
+	}
+}
+
 
 CpuAttributes&
 CpuAttributes::operator+=( CpuAttributes& rhs )
@@ -1313,12 +1528,19 @@ bool
 AvailAttributes::decrement( CpuAttributes* cap ) 
 {
 	int new_cpus, new_phys_mem;
-	float new_virt_mem, new_disk, floor = -0.000001f;
+	double new_virt_mem, new_disk, floor = -0.000001f;
 	
-	new_cpus = a_num_cpus - cap->c_num_cpus;
+	new_cpus = a_num_cpus;
+	if ( ! IS_AUTO_SHARE(cap->c_num_cpus)) {
+		if (cap->c_allow_fractional_cpus && (cap->c_num_cpus < .5)) {
+			//PRAGMA_REMIND("TJ: account for fractional cpus properly...")
+		} else {
+			new_cpus -= cap->c_num_cpus;
+		}
+	}
 
 	new_phys_mem = a_phys_mem;
-	if( cap->c_phys_mem != AUTO_MEM ) {
+	if ( ! IS_AUTO_SHARE(cap->c_phys_mem)) {
 		new_phys_mem -= cap->c_phys_mem;
 	}
 
@@ -1350,9 +1572,12 @@ AvailAttributes::decrement( CpuAttributes* cap )
     }
 
     a_num_cpus = new_cpus;
+    if (IS_AUTO_SHARE(cap->c_num_cpus)) {
+        a_num_cpus_auto_count += 1;
+    }
 
     a_phys_mem = new_phys_mem;
-    if( cap->c_phys_mem == AUTO_MEM ) {
+    if (IS_AUTO_SHARE(cap->c_phys_mem)) {
         a_phys_mem_auto_count += 1;
     }
 
@@ -1376,10 +1601,49 @@ AvailAttributes::decrement( CpuAttributes* cap )
 	return true;
 }
 
-bool
-AvailAttributes::computeAutoShares( CpuAttributes* cap )
+// in order to consume all of a user-defined resource and distribute it among slots,
+// we have to keep track of how many slots have auto shares and how much of a resource
+// remains that is to be used by auto shares.  To do this, we need to capture the
+// auto capacity and slot count into variables that can be decremented as we hand out
+// auto shares in the function below this one.
+//
+bool AvailAttributes::computeRemainder(slotres_map_t & remain_cap, slotres_map_t & remain_cnt)
 {
-	if( cap->c_phys_mem == AUTO_MEM ) {
+	remain_cap.clear();
+	remain_cnt.clear();
+	for (slotres_map_t::iterator j(a_slotres_map.begin());  j != a_slotres_map.end();  ++j) {
+		remain_cap[j->first] = j->second;
+	}
+	for (slotres_map_t::iterator j(a_autocnt_map.begin());  j != a_autocnt_map.end();  ++j) {
+		remain_cnt[j->first] = j->second;
+	}
+	return true;
+}
+
+// hand out cpu, mem, disk, swap and user defined resource auto shares
+// user defined resource auto shares are handed out from the count of resources
+// and auto sharing slots passed in.
+// Note: that remain_cap and remain_cnt are modified by this method,
+//  the initial value of remain_cap and remain_cnt are calculated by the computeRemainder
+// method of this class.  It is the callers responsiblity to call computerRemaineder and
+// to pass the result into this function.
+//
+bool
+AvailAttributes::computeAutoShares( CpuAttributes* cap, slotres_map_t & remain_cap, slotres_map_t & remain_cnt)
+{
+	if (IS_AUTO_SHARE(cap->c_num_cpus)) {
+		ASSERT( a_num_cpus_auto_count > 0 );
+		double new_value = a_num_cpus / a_num_cpus_auto_count;
+		if (cap->c_allow_fractional_cpus) {
+			if ( new_value <= 0.0)
+				return false;
+		} else if( new_value < 1 ) {
+			return false;
+		}
+		cap->c_num_slot_cpus = cap->c_num_cpus = new_value;
+	}
+
+	if (IS_AUTO_SHARE(cap->c_phys_mem)) {
 		ASSERT( a_phys_mem_auto_count > 0 );
 		int new_value = a_phys_mem / a_phys_mem_auto_count;
 		if( new_value < 1 ) {
@@ -1390,7 +1654,7 @@ AvailAttributes::computeAutoShares( CpuAttributes* cap )
 
 	if( IS_AUTO_SHARE(cap->c_virt_mem_fraction) ) {
 		ASSERT( a_virt_mem_auto_count > 0 );
-		float new_value = a_virt_mem_fraction / a_virt_mem_auto_count;
+		double new_value = a_virt_mem_fraction / a_virt_mem_auto_count;
 		if( new_value <= 0 ) {
 			return false;
 		}
@@ -1400,24 +1664,37 @@ AvailAttributes::computeAutoShares( CpuAttributes* cap )
 	if( IS_AUTO_SHARE(cap->c_disk_fraction) ) {
 		AvailDiskPartition &partition = GetAvailDiskPartition( cap->c_execute_partition_id );
 		ASSERT( partition.m_auto_count > 0 );
-		float new_value = partition.m_disk_fraction / partition.m_auto_count;
+		double new_value = partition.m_disk_fraction / partition.m_auto_count;
 		if( new_value <= 0 ) {
 			return false;
 		}
 		cap->c_disk_fraction = new_value;
 	}
 
-    for (slotres_map_t::iterator j(a_slotres_map.begin());  j != a_slotres_map.end();  ++j) {
-        if (IS_AUTO_SHARE(cap->c_slotres_map[j->first])) {
-            // replace auto share vals with final values:
-            ASSERT(a_autocnt_map[j->first] > 0);
-            int new_value = int(j->second / a_autocnt_map[j->first]);
-            if (new_value <= 0) return false;
-            cap->c_slotres_map[j->first] = new_value;
-        }
-        // save off slot resource totals once we have final value:
-        cap->c_slottot_map[j->first] = cap->c_slotres_map[j->first];
-    }
+	for (slotres_map_t::iterator j(a_slotres_map.begin());  j != a_slotres_map.end();  ++j) {
+		if (IS_AUTO_SHARE(cap->c_slotres_map[j->first])) {
+			// replace auto share vals with final values:
+			// if there are less than 1 of the resource remaining, hand out 0
+			ASSERT(a_autocnt_map[j->first] > 0) // since this is auto, we expect that the count of autos is at least 1
+			if (j->second < 1) {
+				cap->c_slotres_map[j->first] = 0;
+			} else {
+				#if 1
+				// distribute integral values evenly among slots, with excess going to first slots(s)
+				long long new_value = (long long)ceil(remain_cap[j->first] / remain_cnt[j->first]);
+				#else
+				// distribute integral values evenly among slots with excess being unused
+				int new_value = int(j->second / a_autocnt_map[j->first]);
+				#endif
+				if (new_value < 0) return false;
+				cap->c_slotres_map[j->first] = new_value;
+				remain_cap[j->first] -= cap->c_slotres_map[j->first];
+			}
+			remain_cnt[j->first] -= 1;
+		}
+		// save off slot resource totals once we have final value:
+		cap->c_slottot_map[j->first] = cap->c_slotres_map[j->first];
+	}
 
 	return true;
 }
