@@ -30,6 +30,32 @@
 #include "cod_mgr.h"
 #include "IdDispenser.h"
 
+#include <set>
+
+class SlotType
+{
+public:
+	typedef std::map<std::string, std::string, classad::CaseIgnLTStr> slottype_param_map_t;
+
+	const char * type_param(const char * name);
+	const char * Shares() { return shares.empty() ? NULL : shares.c_str(); }
+
+	static const char * type_param(CpuAttributes* p_attr, const char * name);
+	static const char * type_param(int type_id, const char * name);
+	static bool type_param_boolean(CpuAttributes* p_attr, const char * name, bool def_value);
+	static bool type_param_boolean(int type_id, const char * name, bool def_value);
+	static long long type_param_long(CpuAttributes* p_attr, const char * name, long long def_value);
+	static char * param(CpuAttributes* p_attr, const char * name);
+	static const char * param(std::string& out, CpuAttributes* p_attr, const char * name);
+	static void init_types(int max_type_id);
+
+private:
+	std::string shares; // share info from SLOT_TYPE_n attribute
+	slottype_param_map_t params;
+	static std::vector<SlotType> types;
+	static bool insert_param(void*, HASHITER & it);
+	void clear() { shares.clear(); params.clear(); }
+};
 
 class Resource : public Service
 {
@@ -37,6 +63,11 @@ public:
 	Resource( CpuAttributes*, int, bool multiple_slots, Resource* _parent = NULL);
 	~Resource();
 
+		// override param by slot_type
+	char * param(const char * name);
+	const char * param(std::string& out, const char * name);
+	const char * param(std::string& out, const char * name, const char * def);
+
 		// Public methods that can be called from command handlers
 	int		retire_claim( bool reversible=false );	// Gracefully finish job and release claim
 	void	void_retire_claim( ) { (void)retire_claim(false); }
@@ -113,6 +144,8 @@ public:
     void	publishDeathTime( ClassAd* cap );
 	void	publishSlotAttrs( ClassAd* );
 	void	refreshSlotAttrs( void );
+	void	publishDynamicChildSummaries( ClassAd *cap);
+	void    rollupDynamicAttrs(ClassAd *cap, std::string &name) const;
 
 		// Load Average related methods
 	float	condor_load( void ) {return r_attr->condor_load();};
@@ -127,8 +160,8 @@ public:
 		// dprintf() functions add the CPU id to the header of each
 		// message for SMP startds (single CPU machines get no special
 		// header and it works just like regular dprintf())
-	void	dprintf( int, const char*, ... );
-	void	dprintf_va( int, const char*, va_list );
+	void	dprintf( int, const char*, ... ) const;
+	void	dprintf_va( int, const char*, va_list ) const;
 
 		// Helper functions to log that we're ignoring a command that
 		// came in while we were in an unexpected state, or while
@@ -173,6 +206,7 @@ public:
 	void	init_classad( void );		
 	void	refresh_classad( amask_t mask );	
 	void	reconfig( void );
+	void	publish_slot_config_overrides(ClassAd * cad);
 
 	void	update( void );		// Schedule to update the central manager.
 	void		do_update( void );			// Actually update the CM
@@ -243,6 +277,17 @@ public:
 	Claim*			r_cur;		// Info about the current claim
 	Claim*			r_pre;		// Info about the possibly preempting claim
 	Claim*			r_pre_pre;	// Info about the preempting preempting claim
+
+    // store multiple claims (currently > 1 for consumption policies)
+    struct claimset_less {
+        bool operator()(Claim* a, Claim* b) const {
+            return strcmp(a->id(), b->id()) < 0;
+        }
+    };
+    typedef std::set<Claim*, claimset_less> claims_t;
+    claims_t        r_claims;
+    bool            r_has_cp;
+
 	CODMgr*			r_cod_mgr;	// Object to manage COD claims
 	Reqexp*			r_reqexp;   // Object for the requirements expression
 	CpuAttributes*	r_attr;		// Attributes of this resource
@@ -251,9 +296,11 @@ public:
 	int				r_id;		// CPU id of this resource (int form)
 	int				r_sub_id;	// Sub id of this resource (int form)
 	char*			r_id_str;	// CPU id of this resource (string form)
+	char*			r_pair_name; // Name of the resource paired with this one, NULL is no pair (the default), may contain "#type" during the slot building process
 	AvailStats		r_avail_stats; // computes resource availability stats
 	int             prevLHF;
 	bool 			m_bUserSuspended;
+	bool			r_no_collector_updates;
 
 	int				type( void ) { return r_attr->type(); };
 
@@ -274,6 +321,13 @@ public:
 	ResourceFeature	get_feature( void ) { return m_resource_feature; }
 
 	void set_parent( Resource* rip );
+    Resource* get_parent() { return m_parent; }
+
+	std::string makeChildClaimIds();
+	void add_dynamic_child(Resource *rip) { m_children.insert(rip); }
+	void remove_dynamic_child(Resource *rip) {m_children.erase(rip); }
+
+	static bool swap_claims(Resource* ripa, Resource* ripb);
 
 	std::list<int> *get_affinity_set() { return &m_affinity_mask;}
 private:
@@ -281,13 +335,19 @@ private:
 
 	Resource*	m_parent;
 
+	// Only partitionable slots have children
+	
+    struct ResourceLess {
+        bool operator()(Resource *lhs, Resource *rhs) const {
+            return strcmp(lhs->r_name, rhs->r_name) < 0;
+        }
+    };
+	std::set<Resource *, ResourceLess> m_children;
+
 	IdDispenser* m_id_dispenser;
 
 	int			update_tid;	// DaemonCore timer id for update delay
-	unsigned	update_sequence;	// Update sequence number
 
-	int		fast_shutdown;	// Flag set if we're in fast shutdown mode.
-	bool    peaceful_shutdown;
 	int		r_cpu_busy;
 	time_t	r_cpu_busy_start_time;
 	time_t	r_last_compute_condor_load;
@@ -344,5 +404,4 @@ The job may be rejected, in which case the returned Resource will be null.
 */
 Resource * initialize_resource(Resource * rip, ClassAd * req_classad, Claim* &leftover_claim);
 
-
 #endif /* _STARTD_RESOURCE_H */
