@@ -24,6 +24,10 @@
 #include "startd.h"
 #include "vm_common.h"
 #include "ipv6_hostname.h"
+#include "consumption_policy.h"
+
+#include <map>
+using std::map;
 
 /* XXX fix me */
 #include "../condor_sysapi/sysapi.h"
@@ -183,6 +187,75 @@ command_activate_claim( Service*, int cmd, Stream* stream )
 	return activate_claim( rip, stream );
 }
 
+// return TRUE on success.
+int swap_claim_and_activation(Resource * rip, ClassAd & opts, Stream* stream)
+{
+	int rval = NOT_OK;
+	Resource* rip_dest = NULL;
+	std::string idd;
+	if (opts.EvalString("DestinationSlotName", rip->r_cur->ad(), idd)) {
+		rip_dest = resmgr->get_by_name(idd.c_str());
+	} else if (opts.EvalString("DestinationClaimId", rip->r_cur->ad(), idd)) {
+		rip_dest = resmgr->get_by_cur_id(idd.c_str());
+	}
+
+	if (rip == rip_dest) {
+		rval = SWAP_CLAIM_ALREADY_SWAPPED; // trivial success, the source and destination were the same.
+	} else if ( ! rip_dest) {
+		dprintf(D_ALWAYS|D_FAILURE, "Destination slot not found when Swapping claims from %s to %s\n", rip->r_name, idd.c_str());
+		rval = NOT_OK;
+	} else if ( ! rip_dest->r_pair_name || MATCH != strcmp(rip_dest->r_pair_name, rip->r_name))  {
+		dprintf(D_ALWAYS|D_FAILURE, "Destination slot not valid when Swapping claims from %s to %s\n", rip->r_name, idd.c_str());
+		rval = NOT_OK;
+	} else if ( ! param_boolean("ALLOW_SLOT_PAIRING", false)) {
+		dprintf(D_ALWAYS, "Ignoring request to swap claims because ALLOW_SLOT_PAIRING is false\n");
+		rval = NOT_OK;
+	} else {
+		dprintf(D_FULLDEBUG, "Swapping claims from %s to %s\n", rip->r_name, rip_dest->r_name);
+		bool swapped = Resource::swap_claims(rip, rip_dest);
+		if ( ! swapped) {
+			dprintf(D_ALWAYS|D_FAILURE, "Failed swap claims from %s to %s\n", rip->r_name, rip_dest->r_name);
+		} else {
+			dprintf(D_ALWAYS, "Claim swap from %s to %s succeeded, updating ads\n", rip->r_id_str, rip_dest->r_id_str);
+
+			// Update the resource classads
+			rip->r_cur->publish( rip->r_classad, A_PUBLIC );
+			rip_dest->r_cur->publish( rip_dest->r_classad, A_PUBLIC );
+			rval = OK;
+		}
+	}
+
+	if (stream) { reply( stream, rval ); } // can remove the if when you remove the hacky testing code.
+	return FALSE; // don't return keep stream.
+}
+
+// handles commands that have a claim id, & classad
+//
+int
+command_with_opts_handler( Service*, int cmd, Stream* stream )
+{
+	int rval = FALSE;
+	ClassAd opts;
+	Resource* rip = stream_to_rip(stream, &opts);
+	if ( ! rip ) {
+		dprintf(D_ALWAYS, "Error: problem finding resource for %d (%s)\n", cmd, getCommandString(cmd));
+		return FALSE;
+	}
+
+	// The rest of these only make sense in claimed state
+	State s = rip->state();
+	if (s != claimed_state) {
+		rip->log_ignore(cmd, s);
+		return FALSE;
+	}
+
+	switch( cmd ) {
+	case SWAP_CLAIM_AND_ACTIVATION:
+		rval = swap_claim_and_activation(rip, opts, stream);
+		break;
+	}
+	return rval;
+}
 
 int
 command_vacate_all( Service*, int cmd, Stream* ) 
@@ -618,6 +691,40 @@ command_match_info( Service*, int cmd, Stream* stream )
 	return rval;
 }
 
+#if 0 // temporary code for testing swap_claim_and_activation
+void hack_test_claim_swap(StringList & args)
+{
+	args.rewind();
+	const char * ida = NULL;
+	const char * idb = NULL;
+	while ((idb = args.next())) {
+		if ( ! ida) ida = idb;
+		else break;
+	}
+	dprintf(D_ALWAYS, "Got command to swap claims for '%s' and '%s'\n", ida ? ida : "NULL", idb ? idb : "NULL");
+	if (ida && idb) {
+		Resource* ripa = resmgr->get_by_name(ida);
+		if ( ! ripa) {
+			dprintf(D_ALWAYS, "Could not find Resource for '%s'\n", ida);
+		} else {
+			const char * pair = ripa->r_pair_name;
+			bool pair_matches = MATCH == strcmp(pair, idb);
+			bool self_matches = MATCH == strcmp(ripa->r_name, idb);
+			dprintf(D_ALWAYS, "Found Resource for '%s', it's pair is '%s' request matches %s\n", 
+				ida, pair ? pair : "NULL", 
+				pair_matches ? "pair" : (self_matches ? "self" : "NO MATCH")
+				);
+
+			ClassAd opts;
+			opts.InsertAttr("DestinationSlotName", idb);
+			dprintf(D_ALWAYS, "calling swap_claim_and_activation\n");
+			int iret = swap_claim_and_activation(ripa, opts, NULL);
+			dprintf(D_ALWAYS, "swap_claim_and_activation returned %d\n", iret);
+		}
+	}
+}
+#endif
+
 
 int
 command_query_ads( Service*, int, Stream* stream) 
@@ -636,14 +743,21 @@ command_query_ads( Service*, int, Stream* stream)
 		return FALSE;
 	}
 
-#if !defined(WANT_OLD_CLASSADS)
-	queryAd.AddExplicitTargetRefs();
+#if defined(ADD_TARGET_SCOPING)
+	AddExplicitTargetRefs( queryAd );
 #endif
 
    MyString stats_config;
    int      dc_publish_flags = daemonCore->dc_stats.PublishFlags;
    queryAd.LookupString("STATISTICS_TO_PUBLISH",stats_config);
    if ( ! stats_config.IsEmpty()) {
+#if 0 // HACK to test swapping claims without a schedd
+       dprintf(D_ALWAYS, "Got QUERY_STARTD_ADS with stats config: %s\n", stats_config.c_str());
+       if (starts_with_ignore_case(stats_config.c_str(), "swap:")) {
+		   StringList swap_args(stats_config.c_str()+5);
+		   hack_test_claim_swap(swap_args);
+       } else
+#endif
       daemonCore->dc_stats.PublishFlags = 
          generic_stats_ParseConfigString(stats_config.Value(), 
                                          "DC", "DAEMONCORE", 
@@ -1080,7 +1194,7 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		ABORT;
 	}
 
-#if !defined(WANT_OLD_CLASSADS)
+#if defined(ADD_TARGET_SCOPING)
 	req_classad->AddTargetRefs( TargetMachineAttrs );
 #endif
 
@@ -1102,6 +1216,29 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		claim->client()->setaddr( client_addr );
 		free( client_addr );
 		client_addr = NULL;
+			// The schedd is asking us to preempt these claims to make
+			// the pslot it is really claiming bigger.  New in 8.1.6
+		int num_preempting = 0;
+		if (stream->code(num_preempting)) {
+			rip->dprintf(D_FULLDEBUG, "Schedd sending %d preempting claims.\n", num_preempting);
+			char **claims = (char **)malloc(sizeof(char *) * (num_preempting));
+			for (int i = 0; i < num_preempting; i++) {
+				claims[i] = NULL;
+				if (! stream->code(claims[i])) {
+					rip->dprintf( D_ALWAYS, "Can't receive preempting claim\n" );
+					ABORT;
+				}
+				Resource *dslot = resmgr->get_by_any_id( claims[i] );
+				if( !dslot ) {
+					ClaimIdParser idp( claims[i] );
+					dprintf( D_ALWAYS, 
+							 "Error: can't find resource with ClaimId (%s)\n", idp.publicClaimId() );
+				}
+				dslot->kill_claim();
+				free(claims[i]);
+			}
+			free(claims);
+		}
 	} else {
 		rip->dprintf(D_FULLDEBUG, "Schedd using pre-v6.1.11 claim protocol\n");
 	}
@@ -1131,6 +1268,13 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		refuse(stream);
 		ABORT;
 	}
+
+    consumption_map_t consumption;
+    bool has_cp = cp_supports_policy(*rip->r_classad);
+    if (has_cp) {
+        cp_override_requested(*req_classad, *rip->r_classad, consumption);
+    }
+
 	if( new_rip != rip) { new_dynamic_slot = true; }
 	rip = new_rip;
 
@@ -1141,6 +1285,10 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		ABORT;
 	}
 
+    if (has_cp) {
+        cp_restore_requested(*req_classad, consumption);
+    }
+
 		// Now, make sure it's got a high enough rank to preempt us.
 	rank = rip->compute_rank(req_classad);
 	rip->dprintf( D_FULLDEBUG, "Rank of this claim is: %f\n", rank );
@@ -1221,10 +1369,13 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 				if( rank > rip->r_cur->rank() ) {
 					rip->dprintf( D_ALWAYS, 
 					 "State change: preempting claim based on machine rank\n" );
+					 resmgr->startd_stats.total_rank_preemptions += 1;
 				} else {
 					rip->dprintf( D_ALWAYS, 
 					 "State change: preempting claim based on user priority\n" );
+					 resmgr->startd_stats.total_user_prio_preemptions += 1;
 				}
+				resmgr->startd_stats.total_preemptions += 1;
 
 				    // Force resource to take note of the preempting claim.
 				    // This results in a reversible transition to the
@@ -1260,6 +1411,27 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		ABORT;
 	}
 
+
+	// if we are claiming this resource, also claim the buddy
+	bool and_pair = false;
+	if (rip->r_pair_name) {
+		Resource * ripb = resmgr->get_by_name(rip->r_pair_name);
+		if (ripb) {
+			req_classad->LookupBool("_condor_SEND_PAIRED_SLOT",and_pair);
+			// we did this to the main claim already, now we need to copy
+			// them to the buddy claim.
+			ripb->r_cur->setaliveint( claim->getaliveint() );
+			ripb->r_cur->client()->setaddr( claim->client()->addr() );
+
+			/* probably don't want to do any of this.
+			ripb->r_cur->setRequestStream( stream );
+			ripb->r_cur->setad( req_classad );
+			ripb->r_cur->setrank( rank );
+			ripb->r_cur->setoldrank( oldrank );
+			*/
+		}
+	}
+
 		// We decided to accept the request, save the schedd's
 		// stream, the rank and the classad of this request.
 	rip->r_cur->setRequestStream( stream );
@@ -1285,7 +1457,7 @@ request_claim( Resource* rip, Claim *claim, char* id, Stream* stream )
 		// function after the preemption has completed when the startd
 		// is finally ready to reply to the and finish the claiming
 		// process.
-	accept_request_claim( rip, leftover_claim );
+	accept_request_claim( rip, leftover_claim, and_pair );
 
 		// We always need to return KEEP_STREAM so that daemon core
 		// doesn't try to delete the stream we've already deleted.
@@ -1324,12 +1496,13 @@ abort_accept_claim( Resource* rip, Stream* stream )
 
 
 bool
-accept_request_claim( Resource* rip, Claim* leftover_claim )
+accept_request_claim( Resource* rip, Claim* leftover_claim, bool and_pair )
 {
 	int interval = -1;
 	char *client_addr = NULL;
 	char RemoteOwner[512];
 	RemoteOwner[0] = '\0';
+	Resource * ripb = NULL;
 
 		// There should not be a pre claim object now.
 	ASSERT( rip->r_pre == NULL );
@@ -1339,18 +1512,30 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 	Sock* sock = (Sock*)stream;
 
 	/* 
-		Reply of 0 (OK) means claim accepted.
-		Reply of 1 (NOT_OK) means claim rejected.
-		Reply of 3 means claim accepted by a partitionable slot,
-	      and the "leftovers" slot ad and claim id will be sent next.
+		Reply of 0 (NOT_OK) means claim rejected.
+		Reply of 1 (OK) means claim accepted.
+		Reply of 3 (REQUEST_CLAIM_LEFTOVERS) means claim accepted by a
+		  partitionable slot, and the "leftovers" slot ad and claim id
+		  will be sent next.
+		Reply of 4 (REQUEST_CLAIM_PAIR) means claim accepted by a slot
+		  that is paired, and the partner slot ad and claim id will be
+		  sent next.
 	*/
 	int cmd = OK;
 	if ( leftover_claim && leftover_claim->id() && 
 		 leftover_claim->rip()->r_classad ) 
 	{
 		// schedd wants leftovers, send reply code 3
-		cmd = 3;
+		cmd = REQUEST_CLAIM_LEFTOVERS;
 	}
+	else if (rip->r_pair_name) {
+		ripb = resmgr->get_by_name(rip->r_pair_name);
+		if (ripb && and_pair) {
+			cmd = REQUEST_CLAIM_PAIR;
+		}
+	}
+
+
 	stream->encode();
 	if( !stream->put( cmd ) ) {
 		rip->dprintf( D_ALWAYS, 
@@ -1358,7 +1543,7 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 		abort_accept_claim( rip, stream );
 		return false;
 	}
-	if ( cmd == 3 ) 
+	if ( cmd == REQUEST_CLAIM_LEFTOVERS )
 	{
 		// schedd just claimed a dynamic slot, and it wants
 		// us to send back to the classad and the new claim id for
@@ -1374,6 +1559,21 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 			return false;
 		}
 	}
+	else if (cmd == REQUEST_CLAIM_PAIR)
+	{
+		dprintf(D_FULLDEBUG,"Sending paired slot claim to schedd\n");
+		//PRAGMA_REMIND("remove these next two dprintfs later")
+		//dprintf(D_FULLDEBUG,"\tmain slot claim id is %s\n", rip->r_cur->id());
+		//dprintf(D_FULLDEBUG,"\tpaired slot claim id is %s\n", ripb->r_cur->id());
+		MyString claimId(ripb->r_cur->id());
+		if ( !stream->put(claimId) || ! putClassAd(stream, *ripb->r_classad)) {
+			rip->dprintf( D_ALWAYS,
+				"Can't send paired slot claim & ad to schedd.\n" );
+			abort_accept_claim( rip, stream );
+			return false;
+		}
+	}
+
 	if( !stream->end_of_message() ) {
 		rip->dprintf( D_ALWAYS, "Can't to send eom to schedd.\n" );
 		abort_accept_claim( rip, stream );
@@ -1405,6 +1605,10 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 		}
 		stream->end_of_message();
 
+		if (ripb) {
+			ripb->r_cur->setaliveint(interval);
+			ripb->r_cur->client()->setaddr(client_addr);
+		}
 			// Now, store them into r_cur
 		rip->r_cur->setaliveint( interval );
 		rip->r_cur->client()->setaddr( client_addr );
@@ -1419,8 +1623,10 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 		MyString ip = sock->peer_addr().to_ip_string();
 		rip->dprintf( D_FULLDEBUG,
 					  "Can't find hostname of client machine %s\n", ip.Value() );
+		if (ripb) { ripb->r_cur->client()->sethost(ip.Value()); }
 		rip->r_cur->client()->sethost(ip.Value());
 	} else {
+		if (ripb) { ripb->r_cur->client()->sethost(hostname.Value()); }
 		rip->r_cur->client()->sethost( hostname.Value() );
 	}
 
@@ -1433,6 +1639,7 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 		RemoteOwner[0] = '\0';
 	}
 	if( '\0' != RemoteOwner[0] ) {
+		if (ripb) { ripb->r_cur->client()->setowner( RemoteOwner ); ripb->r_cur->client()->setuser( RemoteOwner ); }
 		rip->r_cur->client()->setowner( RemoteOwner );
 			// For now, we say the remote user is the same as the
 			// remote owner.  In the future, we might decide to leave
@@ -1447,6 +1654,7 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 	char* acct_grp = NULL;
 	rip->r_cur->ad()->LookupString( ATTR_ACCOUNTING_GROUP, &acct_grp );
 	if( acct_grp ) {
+		if (ripb) { ripb->r_cur->client()->setAccountingGroup( acct_grp ); }
 		rip->r_cur->client()->setAccountingGroup( acct_grp );
 		free( acct_grp );
 		acct_grp = NULL;
@@ -1457,8 +1665,9 @@ accept_request_claim( Resource* rip, Claim* leftover_claim )
 		// Since we're done talking to this schedd, delete the stream.
 	rip->r_cur->setRequestStream( NULL );
 
-	rip->dprintf( D_FAILURE|D_ALWAYS, "State change: claiming protocol successful\n" );
+	rip->dprintf( D_ALWAYS, "State change: claiming protocol successful\n" );
 	rip->change_state( claimed_state );
+	if (ripb) { ripb->change_state( claimed_state ); }
 	return true;
 }
 
@@ -1522,7 +1731,7 @@ activate_claim( Resource* rip, Stream* stream )
 		ABORT;
 	}
 
-#if !defined(WANT_OLD_CLASSADS)
+#if defined(ADD_TARGET_SCOPING)
 	req_classad->AddTargetRefs( TargetMachineAttrs );
 #endif
 
@@ -1548,12 +1757,20 @@ activate_claim( Resource* rip, Stream* stream )
 		// See if machine and job meet each other's requirements, if
 		// so start the job and tell shadow, otherwise refuse and
 		// clean up.  
+    consumption_map_t consumption;
+    bool has_cp = cp_supports_policy(*mach_classad, false);
+    bool cp_sufficient = true;
+    if (has_cp) {
+        cp_override_requested(*req_classad, *mach_classad, consumption);
+        cp_sufficient = cp_sufficient_assets(*mach_classad, consumption);
+    }
+
 	rip->r_reqexp->restore();
 	if( mach_classad->EvalBool( ATTR_REQUIREMENTS, 
 								req_classad, mach_requirements ) == 0 ) {
 		mach_requirements = 0;
 	}
-	if( !mach_requirements ) {
+	if (!(cp_sufficient && mach_requirements)) {
 		rip->dprintf( D_ALWAYS, "Machine Requirements check failed!\n" );
 		refuse( stream );
 	    ABORT;
@@ -1585,6 +1802,11 @@ activate_claim( Resource* rip, Stream* stream )
 												   mach_classad,
 												   no_starter,
 												   starter );
+
+    if (has_cp) {
+        cp_restore_requested(*req_classad, consumption);
+    }
+
 	if( ! tmp_starter ) {
 		if( no_starter ) {
 			rip->dprintf( D_ALWAYS, "No valid starter found to run this job!  Is something wrong with your Condor installation?\n" );
@@ -1736,7 +1958,7 @@ activate_claim( Resource* rip, Stream* stream )
 		// Finally, update all these things into the resource classad.
 	rip->r_cur->publish( rip->r_classad, A_PUBLIC );
 
-	rip->dprintf( D_FAILURE|D_ALWAYS, 
+	rip->dprintf( D_ALWAYS,
 				  "State change: claim-activation protocol successful\n" );
 	rip->change_state( busy_act );
 
@@ -1791,7 +2013,7 @@ match_info( Resource* rip, char* id )
 #endif /* HAVE_BACKFILL */
 	case unclaimed_state:
 	case owner_state:
-		if( rip->r_cur->idMatches(id) ) {
+        if (rip->r_cur->idMatches(id)) {
 			rip->dprintf( D_ALWAYS, "Received match %s\n", idp.publicClaimId() );
 
 			if( rip->destination_state() != no_state ) {
@@ -1809,7 +2031,7 @@ match_info( Resource* rip, char* id )
 				// too long. 
 			rip->r_cur->start_match_timer();
 
-			rip->dprintf( D_FAILURE|D_ALWAYS, "State change: "
+			rip->dprintf( D_ALWAYS, "State change: "
 						  "match notification protocol successful\n" );
 
 #if HAVE_BACKFILL
@@ -2080,7 +2302,7 @@ command_classad_handler( Service*, int dc_cmd, Stream* s )
 		cmd = getCmdFromReliSock( rsock, &ad, false );
 	}
 
-#if !defined(WANT_OLD_CLASSADS)
+#if defined(ADD_TARGET_SCOPING)
 	ad.AddTargetRefs( TargetMachineAttrs );
 #endif
 
