@@ -80,11 +80,17 @@ struct SortSpec {
 
 // global variables
 AttrListPrintMask pm;
+printmask_headerfooter_t pmHeadFoot = STD_HEADFOOT;
 List<const char> pm_head; // The list of headings for the mask entries
+std::vector<GroupByKeyInfo> group_by_keys; // TJ 8.1.5 for future use, ignored for now.
+bool explicit_format = false;
+bool using_print_format = false; // hack for now so we can get standard totals when using -print-format
+bool disable_user_print_files = false; // allow command line to defeat use of default user print files.
 const char		*DEFAULT= "<default>";
 DCCollector* pool = NULL;
 AdTypes		type 	= (AdTypes) -1;
 ppOption	ppStyle	= PP_NOTSET;
+ppOption	ppTotalStyle = PP_NOTSET; // used when setting PP_CUSTOM to keep track of how to do totals.
 int			wantOnlyTotals 	= 0;
 int			summarySize = -1;
 bool        expert = false;
@@ -134,6 +140,7 @@ main (int argc, char *argv[])
 	myDistro->Init( argc, argv );
 	myName = argv[0];
 	config();
+	dprintf_config_tool_on_error(0);
 
 	// The arguments take two passes to process --- the first pass
 	// figures out the mode, after which we can instantiate the required
@@ -148,6 +155,7 @@ main (int argc, char *argv[])
 
 	// instantiate query object
 	if (!(query = new CondorQuery (type))) {
+		dprintf_WriteOnErrorBuffer(stderr, true);
 		fprintf (stderr, "Error:  Out of memory\n");
 		exit (1);
 	}
@@ -163,6 +171,11 @@ main (int argc, char *argv[])
 		break;
 #endif /* HAVE_EXT_POSTGRESQL */
 
+
+	  case DEFRAG_AD:
+		setPPstyle(PP_GENERIC_NORMAL, 0, DEFAULT);
+		break;
+
 	  case STARTD_AD:
 		setPPstyle(PP_STARTD_NORMAL, 0, DEFAULT);
 		break;
@@ -213,6 +226,7 @@ main (int argc, char *argv[])
 	  case MODE_QUILL_NORMAL:
 #endif /* HAVE_EXT_POSTGRESQL */
 
+	  case MODE_DEFRAG_NORMAL:
 	  case MODE_STARTD_NORMAL:
 	  case MODE_MASTER_NORMAL:
 	  case MODE_CKPT_SRVR_NORMAL:
@@ -320,7 +334,12 @@ main (int argc, char *argv[])
 	secondPass (argc, argv);
 
 	// initialize the totals object
-	TrackTotals	totals(ppStyle);
+	if (ppStyle == PP_CUSTOM && using_print_format) {
+		if (pmHeadFoot & HF_NOSUMMARY) ppTotalStyle = PP_CUSTOM;
+	} else {
+		ppTotalStyle = ppStyle;
+	}
+	TrackTotals	totals(ppTotalStyle);
 
 	// fetch the query
 	QueryResult q;
@@ -428,11 +447,12 @@ main (int argc, char *argv[])
 				addr = d->addr();
 				requested_daemon = d;
 			} else {
-			        const char* id = d->idStr();
-                                if (NULL == id) id = d->name();
+				const char* id = d->idStr();
+				if (NULL == id) id = d->name();
+				dprintf_WriteOnErrorBuffer(stderr, true);
 				if (NULL == id) id = "daemon";
-           	                fprintf(stderr, "Error: Failed to locate %s\n", id);
-                                fprintf(stderr, "%s\n", d->error());
+				fprintf(stderr, "Error: Failed to locate %s\n", id);
+				fprintf(stderr, "%s\n", d->error());
 				exit( 1 );
 			}
 		}
@@ -440,13 +460,13 @@ main (int argc, char *argv[])
 
 	CondorError errstack;
 	if (NULL != addr) {
-	        // this case executes if pool was provided, or if in "direct" mode with
-	        // subsystem that corresponds to a daemon (above).
-                // Here 'addr' represents either the host:port of requested pool, or
-                // alternatively the host:port of daemon associated with requested subsystem (direct mode)
+			// this case executes if pool was provided, or if in "direct" mode with
+			// subsystem that corresponds to a daemon (above).
+			// Here 'addr' represents either the host:port of requested pool, or
+			// alternatively the host:port of daemon associated with requested subsystem (direct mode)
 		q = query->fetchAds (result, addr, &errstack);
 	} else {
-                // otherwise obtain list of collectors and submit query that way
+			// otherwise obtain list of collectors and submit query that way
 		CollectorList * collectors = CollectorList::create();
 		q = collectors->query (*query, result, &errstack);
 		delete collectors;
@@ -454,32 +474,36 @@ main (int argc, char *argv[])
 		
 
 	// if any error was encountered during the query, report it and exit 
-        if (Q_OK != q) {
-                // we can always provide these messages:
-	        fprintf( stderr, "Error: %s\n", getStrQueryResult(q) );
+	if (Q_OK != q) {
+
+		dprintf_WriteOnErrorBuffer(stderr, true);
+			// we can always provide these messages:
+		fprintf( stderr, "Error: %s\n", getStrQueryResult(q) );
 		fprintf( stderr, "%s\n", errstack.getFullText(true).c_str() );
 
-	        if ((NULL != requested_daemon) && ((Q_NO_COLLECTOR_HOST == q) || (requested_daemon->type() == DT_COLLECTOR))) {
-                        // Specific long message if connection to collector failed.
-		        const char* fullhost = requested_daemon->fullHostname();
-                        if (NULL == fullhost) fullhost = "<unknown_host>";
-                        const char* daddr = requested_daemon->addr();
-                        if (NULL == daddr) daddr = "<unknown>";
-                        char info[1000];
-                        sprintf(info, "%s (%s)", fullhost, daddr);
-		        printNoCollectorContact( stderr, info, !expert );                        
-	        } else if ((NULL != requested_daemon) && (Q_COMMUNICATION_ERROR == q)) {
-                        // more helpful message for failure to connect to some daemon/subsys
+        if ((NULL != requested_daemon) && ((Q_NO_COLLECTOR_HOST == q) ||
+			(requested_daemon->type() == DT_COLLECTOR)))
+		{
+				// Specific long message if connection to collector failed.
+			const char* fullhost = requested_daemon->fullHostname();
+			if (NULL == fullhost) fullhost = "<unknown_host>";
+			const char* daddr = requested_daemon->addr();
+			if (NULL == daddr) daddr = "<unknown>";
+			char info[1000];
+			sprintf(info, "%s (%s)", fullhost, daddr);
+	        printNoCollectorContact( stderr, info, !expert );
+        } else if ((NULL != requested_daemon) && (Q_COMMUNICATION_ERROR == q)) {
+				// more helpful message for failure to connect to some daemon/subsys
 			const char* id = requested_daemon->idStr();
-                        if (NULL == id) id = requested_daemon->name();
+			if (NULL == id) id = requested_daemon->name();
 			if (NULL == id) id = "daemon";
-                        const char* daddr = requested_daemon->addr();
-                        if (NULL == daddr) daddr = "<unknown>";
-           	        fprintf(stderr, "Error: Failed to contact %s at %s\n", id, daddr);
+			const char* daddr = requested_daemon->addr();
+			if (NULL == daddr) daddr = "<unknown>";
+			fprintf(stderr, "Error: Failed to contact %s at %s\n", id, daddr);
 		}
 
-                // fail
-                exit (1);
+		// fail
+		exit (1);
 	}
 
 	if (sortSpecs.empty()) {
@@ -518,6 +542,59 @@ main (int argc, char *argv[])
 	return 0;
 }
 
+const CustomFormatFnTable * getCondorStatusPrintFormats();
+
+int set_status_print_mask_from_stream (
+	const char * streamid,
+	bool is_filename)
+{
+	std::string where_expr;
+	std::string messages;
+	StringList attrs;
+
+	SimpleInputStream * pstream = NULL;
+
+	FILE *file = NULL;
+	if (MATCH == strcmp("-", streamid)) {
+		pstream = new SimpleFileInputStream(stdin, false);
+	} else if (is_filename) {
+		file = safe_fopen_wrapper_follow(streamid, "r");
+		if (file == NULL) {
+			fprintf(stderr, "Can't open select file: %s\n", streamid);
+			return -1;
+		}
+		pstream = new SimpleFileInputStream(file, true);
+	} else {
+		pstream = new StringLiteralInputStream(streamid);
+	}
+	ASSERT(pstream);
+
+	int err = SetAttrListPrintMaskFromStream(
+					*pstream,
+					*getCondorStatusPrintFormats(),
+					pm,
+					pmHeadFoot,
+					group_by_keys,
+					where_expr,
+					attrs,
+					messages);
+	delete pstream; pstream = NULL;
+	if ( ! err) {
+		if ( ! where_expr.empty()) {
+			const char * constraint = pm.store(where_expr.c_str());
+			if (query->addANDConstraint (constraint) != Q_OK) {
+				formatstr_cat(messages, "WHERE expression is not valid: %s\n", constraint);
+			}
+		}
+		// convert projection list into the format that condor status likes. because programmers.
+		attrs.rewind();
+		const char * attr;
+		while ((attr = attrs.next())) { projList.AppendArg(attr); }
+	}
+	if ( ! messages.empty()) { fprintf(stderr, "%s", messages.c_str()); }
+	return err;
+}
+
 
 void
 usage ()
@@ -525,7 +602,8 @@ usage ()
 	fprintf (stderr,"Usage: %s [help-opt] [query-opt] [display-opt] "
 		"[custom-opts ...] [name ...]\n"
 		"    where [help-opt] is one of\n"
-		"\t-help\t\t\tThis screen\n"
+		"\t-help\t\t\tPrint this screen and exit\n"
+		"\t-version\t\tPrint HTCondor version and exit\n"
 		"\t-diagnose\t\tPrint out query ad without performing query\n"
 		"    and [query-opt] is one of\n"
 		"\t-avail\t\t\tPrint information about available resources\n"
@@ -534,6 +612,7 @@ usage ()
 		"\t-cod\t\t\tDisplay Computing On Demand (COD) jobs\n"
 		"\t-collector\t\tDisplay collector daemon attributes\n"
 		"\t-debug\t\t\tDisplay debugging info to console\n"
+		"\t-defrag\t\t\tDisplay status of defrag daemon\n"
 		"\t-direct <host>\t\tGet attributes directly from the given daemon\n"
 		"\t-java\t\t\tDisplay Java-capable hosts\n"
 		"\t-vm\t\t\tDisplay VM-capable hosts\n"
@@ -587,6 +666,7 @@ firstPass (int argc, char *argv[])
 	int had_pool_error = 0;
 	int had_direct_error = 0;
 	int had_statistics_error = 0;
+	//bool explicit_mode = false;
 	const char * pcolon = NULL;
 
 	// Process arguments:  there are dependencies between them
@@ -620,6 +700,7 @@ firstPass (int argc, char *argv[])
 			}
 			pool = new DCCollector( argv[i] );
 			if( !pool->addr() ) {
+				dprintf_WriteOnErrorBuffer(stderr, true);
 				fprintf( stderr, "Error: %s\n", pool->error() );
 				if (!expert) {
 					printf("\n");
@@ -641,7 +722,8 @@ firstPass (int argc, char *argv[])
 				fprintf( stderr, "Use \"%s -help\" for details\n", myName );
 				exit( 1 );
 			}
-			i += 2;			
+			i += 2;
+			explicit_format = true;
 		} else
 		if (*argv[i] == '-' &&
 			(is_arg_colon_prefix(argv[i]+1, "autoformat", &pcolon, 5) || 
@@ -653,6 +735,7 @@ firstPass (int argc, char *argv[])
 				fprintf( stderr, "Use \"%s -help\" for details\n", myName );
 				exit( 1 );
 			}
+			explicit_format = true;
 			setPPstyle (PP_CUSTOM, i, argv[i]);
 			while (argv[i+1] && *(argv[i+1]) != '-') {
 				++i;
@@ -662,6 +745,15 @@ firstPass (int argc, char *argv[])
 				++i;
 			}
 		} else
+		if (is_dash_arg_colon_prefix(argv[i], "print-format", &pcolon, 2)) {
+			if ( (i+1 >= argc)  || (*(argv[i+1]) == '-' && (argv[i+1])[1] != 0)) {
+				fprintf( stderr, "Error: Argument -print-format requires a filename argument\n");
+				exit( 1 );
+			}
+			explicit_format = true;
+			++i; // eat the next argument.
+			// we can't fully parse the print format argument until the second pass, so we are done for now.
+		} else
 		if (matchPrefix (argv[i], "-wide", 3)) {
 			wide_display = true; // when true, don't truncate field data
 			//invalid_fields_empty = true;
@@ -711,6 +803,9 @@ firstPass (int argc, char *argv[])
 			// dprintf to console
 			dprintf_set_tool_debug("TOOL", 0);
 		} else
+		if (matchPrefix (argv[i], "-defrag", 4)) {
+			setMode (MODE_DEFRAG_NORMAL, i, argv[i]);
+		} else
 		if (matchPrefix (argv[i], "-help", 2)) {
 			usage ();
 			exit (0);
@@ -737,13 +832,13 @@ firstPass (int argc, char *argv[])
 			setMode (MODE_STARTD_COD, i, argv[i]);
 		} else
 		if (matchPrefix (argv[i], "-java", 2)) {
-			javaMode = true;
+			/*explicit_mode =*/ javaMode = true;
 		} else
 		if (matchPrefix (argv[i], "-absent", 3)) {
-			absentMode = true;
+			/*explicit_mode =*/ absentMode = true;
 		} else
 		if (matchPrefix (argv[i], "-vm", 3)) {
-			vmMode = true;
+			/*explicit_mode =*/ vmMode = true;
 		} else
 		if (matchPrefix (argv[i], "-server", 3)) {
 			setPPstyle (PP_STARTD_SERVER, i, argv[i]);
@@ -894,6 +989,7 @@ firstPass (int argc, char *argv[])
 		} else
 		if (matchPrefix (argv[i], "-total", 2)) {
 			wantOnlyTotals = 1;
+			explicit_format = true;
 		} else
 		if (matchPrefix(argv[i], "-expert", 2)) {
 			expert = true;
@@ -1035,6 +1131,27 @@ secondPass (int argc, char *argv[])
 			}
 			continue;
 		}
+		if (is_dash_arg_colon_prefix(argv[i], "print-format", &pcolon, 2)) {
+			if ( (i+1 >= argc)  || (*(argv[i+1]) == '-' && (argv[i+1])[1] != 0)) {
+				fprintf( stderr, "Error: Argument -print-format requires a filename argument\n");
+				exit( 1 );
+			}
+			// hack allow -pr ! to disable use of user-default print format files.
+			if (MATCH == strcmp(argv[i+1], "!")) {
+				++i;
+				disable_user_print_files = true;
+				continue;
+			}
+			ppTotalStyle = ppStyle;
+			setPPstyle (PP_CUSTOM, i, argv[i]);
+			++i; // skip to the next argument.
+			if (set_status_print_mask_from_stream(argv[i], true) < 0) {
+				fprintf(stderr, "Error: invalid select file %s\n", argv[i]);
+				exit (1);
+			}
+			using_print_format = true; // so we can hack totals.
+			continue;
+		}
 		if (matchPrefix (argv[i], "-target", 5)) {
 			i++;
 			continue;
@@ -1087,6 +1204,7 @@ secondPass (int argc, char *argv[])
 					// use what we are given and do not flag an error.
 					daemonname = strnewp(argv[i]);
 				} else {
+					dprintf_WriteOnErrorBuffer(stderr, true);
 					fprintf( stderr, "%s: unknown host %s\n",
 								 argv[0], get_host_part(argv[i]) );
 					exit(1);
@@ -1094,6 +1212,7 @@ secondPass (int argc, char *argv[])
 			}
 
 			switch (mode) {
+			  case MODE_DEFRAG_NORMAL:
 			  case MODE_STARTD_NORMAL:
 			  case MODE_STARTD_COD:
 #ifdef HAVE_EXT_POSTGRESQL
