@@ -28,6 +28,7 @@
 #include "util_lib_proto.h"
 #include "classad_merge.h"
 #include "condor_fsync.h"
+#include "condor_attributes.h"
 
 #if defined(HAVE_DLOPEN)
 #include "ClassAdLogPlugin.h"
@@ -54,6 +55,100 @@ ptr = NULL;
 
 const char *EMPTY_CLASSAD_TYPE_NAME = "(empty)";
 
+ClassAdLogFilterIterator::ClassAdLogFilterIterator(ClassAdHashTable *table, const classad::ExprTree *requirements, int timeslice_ms, bool invalid)
+	: m_table(table),
+	  m_cur(table->begin()),
+	  m_found_ad(false),
+	  m_requirements(requirements),
+	  m_timeslice_ms(timeslice_ms),
+	  m_done(invalid)
+	{}
+
+ClassAdLogFilterIterator::ClassAdLogFilterIterator(const ClassAdLogFilterIterator &other)
+	: m_table(other.m_table),
+	  m_cur(other.m_cur),
+	  m_found_ad(other.m_found_ad),
+	  m_requirements(other.m_requirements),
+	  m_timeslice_ms(other.m_timeslice_ms),
+	  m_done(other.m_done)
+{
+}
+
+ClassAd* ClassAdLogFilterIterator::operator *() const {
+	if (m_done || (m_cur == m_table->end()) || !m_found_ad)
+	{
+		return NULL;
+	}
+	return (*m_cur).second;
+}
+
+ClassAdLogFilterIterator
+ClassAdLogFilterIterator::operator++(int)
+{
+	m_found_ad = false;
+	ClassAdLogFilterIterator cur = *this;
+	if (m_done) {
+		return cur;
+	}
+
+	HashIterator<HashKey, ClassAd*> end = m_table->end();
+	bool boolVal;
+	int intVal;
+	int miss_count = 0;
+	while (!(m_cur == end))
+	{
+		miss_count++;
+		if (miss_count == m_timeslice_ms)
+		{
+			break;
+		}
+		cur = *this;
+		ClassAd *tmp_ad = (*m_cur++).second;
+		if (!tmp_ad) continue;
+		if (m_requirements) {
+			classad::ExprTree &requirements = *const_cast<classad::ExprTree*>(m_requirements);
+			const classad::ClassAd *old_scope = requirements.GetParentScope();
+			requirements.SetParentScope( tmp_ad );
+				classad::Value result;
+			int retval = requirements.Evaluate(result);
+			requirements.SetParentScope(old_scope);
+			if (!retval) {
+				dprintf(D_FULLDEBUG, "Unable to evaluate ad.\n");
+				continue;
+			}
+
+			if (!(result.IsBooleanValue(boolVal) && boolVal) &&
+					!(result.IsIntegerValue(intVal) && intVal)) {
+				continue;
+			}
+		}
+		int tmp_int;
+		if (!tmp_ad->EvaluateAttrInt(ATTR_CLUSTER_ID, tmp_int) || !tmp_ad->EvaluateAttrInt(ATTR_PROC_ID, tmp_int)) {
+			continue;
+		}
+                int proc, cluster;
+                tmp_ad->EvaluateAttrInt(ATTR_CLUSTER_ID, cluster);
+                tmp_ad->EvaluateAttrInt(ATTR_PROC_ID, proc);
+		cur.m_found_ad = true;
+		m_found_ad = true;
+		break;
+	}
+	if ((m_cur == end) && (!m_found_ad)) {
+		m_done = true;
+	}
+	return cur;
+}
+
+bool
+ClassAdLogFilterIterator::operator==(const ClassAdLogFilterIterator &other)
+{
+	if (m_table != other.m_table) return false;
+	if (m_done && other.m_done) return true;
+	if (m_done != other.m_done) return false;
+	if (!(m_cur == other.m_cur) ) return false;
+	return true;
+}
+
 ClassAdLog::ClassAdLog() : table(CLASSAD_LOG_HASHTABLE_SIZE, hashFunction)
 {
 	active_transaction = NULL;
@@ -207,14 +302,7 @@ ClassAdLog::AppendLog(LogRecord *log)
 				EXCEPT("write to %s failed, errno = %d", logFilename(), errno);
 			}
 			if( m_nondurable_level == 0 ) {
-					//MD: flushing data -- using a file pointer
-				if (fflush(log_fp) !=0){
-					EXCEPT("flush to %s failed, errno = %d", logFilename(), errno);
-				}
-					//MD: syncing the data as done before
-				if (condor_fsync(fileno(log_fp)) < 0) {
-					EXCEPT("fsync of %s failed, errno = %d", logFilename(), errno);
-				}
+				ForceLog();  // flush and fsync
 			}
 		}
 		log->Play((void *)&table);
@@ -232,6 +320,25 @@ ClassAdLog::FlushLog()
 	}
 }
 
+void
+ClassAdLog::ForceLog()
+{
+	// Force log changes to disk.  This involves first flushing
+	// the log from memory buffers, then fsyncing to disk.
+	if (log_fp!=NULL) {
+
+		// First flush
+		FlushLog();
+
+		// Then sync
+		if (condor_fsync(fileno(log_fp)) < 0) {
+			EXCEPT("fsync of %s failed, errno = %d", logFilename(), errno);
+		}
+
+	}
+}
+
+
 bool
 ClassAdLog::SaveHistoricalLogs()
 {
@@ -1036,6 +1143,15 @@ LogDeleteAttribute::WriteBody(FILE* fp)
 	return rval1 + rval;
 }
 
+int
+LogBeginTransaction::Play(void *){
+#if defined(HAVE_DLOPEN)
+	ClassAdLogPluginManager::BeginTransaction();
+#endif
+
+	return 1;
+}
+
 int 
 LogBeginTransaction::ReadBody(FILE* fp)
 {
@@ -1047,6 +1163,15 @@ LogBeginTransaction::ReadBody(FILE* fp)
 	return( 1 );
 }
 
+int
+LogEndTransaction::Play(void *) {
+#if defined(HAVE_DLOPEN)
+	ClassAdLogPluginManager::EndTransaction();
+#endif
+
+	return 1;
+}
+
 int 
 LogEndTransaction::ReadBody( FILE* fp )
 {
