@@ -19,7 +19,9 @@ BOOST_PYTHON_MODULE(htcondor)
     export_config();
     export_daemon_and_ad_types();
     export_collector();
+    export_negotiator();
     export_schedd();
     export_dc_tool();
     export_secman();
+    export_event_reader();
 }
